searchData={"items":[{"type":"module","title":"Backstage","doc":"","ref":"Backstage.html"},{"type":"function","title":"Backstage.ack/3","doc":"","ref":"Backstage.html#ack/3"},{"type":"function","title":"Backstage.start_link/1","doc":"","ref":"Backstage.html#start_link/1"},{"type":"function","title":"Backstage.transform/2","doc":"","ref":"Backstage.html#transform/2"},{"type":"extras","title":"README","doc":"- [🚧 Backstage 🚧](#-backstage-)\n- [Goals](#goals)\n- [CQRWho?](#cqrwho)\n  - [The Script](#the-script)\n  - [Business Value](#business-value)\n- [Architecture](#architecture)\n- [TODO](#todo)\n- [Installation](#installation)\n\n---\n\n> On Broadway, the backstage is buzzing with activity.\n>\n> The actors are stepping onto the stage.\n>\n> The crew is pulling the curtains, aiming the lights.\n>\n> The band is moving air.\n>\n> The director is observant and diligent.\n>\n> The audience is transfixed.\n>\n> The show must go on, even when mistakes are made.\n\n---\n\n# 🚧 Backstage 🚧\n\nBackstage is a [CQRS & Event Sourcing](#cqrwho) framework optimized for Elixir applications to take advantage of concurrency and parallelism on the BEAM.\n\n> 👨‍🎤 This repository is currently in an **experimentation phase**. It is not intended to be used in any critical production systems.\n>\n> See [hex.pm](https://hex.pm/packages?search=cqrs&sort=recent_downloads) for a list of available CQRS packages.\n\n# Goals\n\n1. Make Event Sourcing more approachable for CRUD developers.\n2. Leverage battle-tested packages from the Elixir ecosystem for maximum performance.\n3. Establish strong patterns for busy development teams.\n\n# CQRWho?\n\n**Command & Query Responsibility Segregation**, or CQRS, is a pattern for separating how data is written to and read from a system. The benefits of this pattern include:\n\n- More control over database performance based on an application's read/write demands\n- Richer data history, useful for auditing, debugging, and AI model training\n- Clearer boundaries for data as it propagates through a system","ref":"readme.html"},{"type":"extras","title":"The Script - README","doc":"For developers familiar with the **Create Read Update Delete** pattern, or CRUD, CQRS may be intimidating at first. We're already seeing new terms and acronyms, which may be unfamiliar. To understand the value of Event Sourcing and CQRS without digging into the specifics, let's think about a script for a Broadway play.\n\nWhether it's authored by a single person or a result of collaboration, a script provides an order of operations for a play. This includes scene-setting and context, lines for characters, music cues, stage direction, and other technical details. This metaphor illustrates how a script provide a repeatable set of written instructions for recreating a performance.\n\nImagine we only had the last page of a script.\n\n- What would we know?\n- What questions could we answer?\n\nWith traditional CRUD database models, we effectively find ourselves with the last page of the script. Our understanding of the system is limited to the data available in its current state. We have limited ability to answer questions which were not anticipated as the data model was designed or as it evolved. While external tools like logging and tracing allow engineers to perform debugging, these tools are not intended nor ideal for business data analysis.","ref":"readme.html#the-script"},{"type":"extras","title":"Business Value - README","doc":"The greatest value delivered by Event Sourcing & CQRS patterns may be its ability to provide answers to unforeseen questions as a business grows and evolves in surprising ways. For engineers focussed on developer operations, these patterns allow more granular control over write and read performance since incoming data (events) is stored separately from data used by application queries (projections).\n\n# Architecture\n\nBackstage uses the [Broadway](https://hexdocs.pm/broadway/introduction.html) package to run highly concurrent and robust data pipelines.\n\n| Component     | Description                                                | Example                 |\n| ------------- | ---------------------------------------------------------- | ----------------------- |\n| Command       | Instructs system to change its state                       | `%AddTicket{}` struct   |\n| Event         | Represents a historical change in the system's state       | `%TicketAdded{}` struct |\n| Router        | Routes commands to aggregates                              | `TicketRouter` module   |\n| Aggregate     | Performs business logic, updates state for a system entity | `Tickets` module        |\n| Event Handler | Executes custom code for matching events                   | `TicketHandler` module  |\n| Projection    | Persists system state to a readable format                 | `tickets` DB table      |\n\n# TODO\n\n- [ ] Investigate Broadway as an Event Broker\n- [ ] Build EventProducer\n- [ ] Investigate protocols for commands, events, and other components\n\n# Installation\n\nTODO","ref":"readme.html#business-value"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]},"content_type":"text/markdown"}